@@ -2,9 +2,11 @@ open Import
 
 let ls_term (fetch_results : Path.Build.t -> string list Action_builder.t) =
   let+ builder = Common.Builder.term
-  and+ paths = Arg.(value & pos_all string [ "." ] & info [] ~docv:"DIR")
+  (* CR-someday Alizter: document this option *)
+  and+ paths = Arg.(value & pos_all string [ "." ] & info [] ~docv:"DIR" ~doc:None)
   and+ context =
-    Common.context_arg ~doc:"The context to look in. Defaults to the default context."
+    Common.context_arg
+      ~doc:(Some "The context to look in. Defaults to the default context.")
   in
   let common, config = Common.init builder in
   let request (_ : Dune_rules.Main.build_system) =
@@ -85,7 +87,7 @@ let ls_term (fetch_results : Path.Build.t -> string list Action_builder.t) =
   Scheduler.go_with_rpc_server ~common ~config
   @@ fun () ->
   let open Fiber.O in
-  Build_cmd.run_build_system ~common ~request
+  Build.run_build_system ~common ~request
   >>| fun (_ : (unit, [ `Already_reported ]) result) -> ()
 ;;
 
