@@ -30,6 +30,24 @@ let display_files files_to_promote =
     Console.printf "%s" (Diff_promotion.File.source file |> Path.Source.to_string))
 ;;
 
+let show_corrected_contents files_to_promote =
+  let open Fiber.O in
+  let files = Diff_promotion.load_db () |> Diff_promotion.filter_db files_to_promote in
+  let+ () = Fiber.return () in
+  List.iter files ~f:(fun file ->
+    let correction_file = Diff_promotion.File.correction_file file in
+    if Path.exists correction_file
+    then (
+      let contents = Io.read_file correction_file in
+      Console.printf "%s" contents)
+    else
+      User_warning.emit
+        [ Pp.textf
+            "Corrected file does not exist for %s."
+            (Diff_promotion.File.source file |> Path.Source.to_string_maybe_quoted)
+        ])
+;;
+
 module Apply = struct
   let info =
     let doc = "Promote files from the last run" in
@@ -52,7 +70,10 @@ module Apply = struct
 
   let term =
     let+ builder = Common.Builder.term
-    and+ files = Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE") in
+    and+ files =
+      (* CR-someday Alizter: document this option *)
+      Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE" ~doc:None)
+    in
     let common, config = Common.init builder in
     let files_to_promote = files_to_promote ~common files in
     match Dune_util.Global_lock.lock ~timeout:None with
@@ -62,16 +83,16 @@ module Apply = struct
         let+ () = Fiber.return () in
         Diff_promotion.promote_files_registered_in_last_run files_to_promote)
     | Error lock_held_by ->
-      Rpc_common.run_via_rpc
-        ~builder
-        ~common
-        ~config
-        lock_held_by
-        (Rpc_common.fire_request
-           ~name:"promote_many"
-           ~wait:true
-           Dune_rpc_private.Procedures.Public.promote_many)
-        files_to_promote
+      Scheduler.go_without_rpc_server ~common ~config (fun () ->
+        let open Fiber.O in
+        Rpc.Rpc_common.fire_request
+          ~name:"promote_many"
+          ~wait:true
+          ~lock_held_by
+          builder
+          Dune_rpc_private.Procedures.Public.promote_many
+          files_to_promote
+        >>| Rpc.Rpc_common.wrap_build_outcome_exn ~print_on_success:true)
   ;;
 
   let command = Cmd.v info term
@@ -82,7 +103,10 @@ module Diff = struct
 
   let term =
     let+ builder = Common.Builder.term
-    and+ files = Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE") in
+    and+ files =
+      (* CR-someday Alizter: document this option *)
+      Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE" ~doc:None)
+    in
     let common, config = Common.init builder in
     let files_to_promote = files_to_promote ~common files in
     Scheduler.go_with_rpc_server ~common ~config (fun () ->
@@ -97,7 +121,10 @@ module Files = struct
 
   let term =
     let+ builder = Common.Builder.term
-    and+ files = Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE") in
+    and+ files =
+      (* CR-someday Alizter: document this option *)
+      Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE" ~doc:None)
+    in
     let common, config = Common.init builder in
     let files_to_promote = files_to_promote ~common files in
     Scheduler.go_with_rpc_server ~common ~config (fun () ->
@@ -107,11 +134,28 @@ module Files = struct
   let command = Cmd.v info term
 end
 
+module Show = struct
+  let info = Cmd.info ~doc:"Display contents of a corrected file" "show"
+
+  let term =
+    let+ builder = Common.Builder.term
+    and+ files =
+      Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE" ~doc:None)
+    in
+    let common, config = Common.init builder in
+    let files_to_promote = files_to_promote ~common files in
+    Scheduler.go_with_rpc_server ~common ~config (fun () ->
+      show_corrected_contents files_to_promote)
+  ;;
+
+  let command = Cmd.v info term
+end
+
 let info =
   Cmd.info ~doc:"Control how changes are propagated back to source code." "promotion"
 ;;
 
-let group = Cmd.group info [ Files.command; Apply.command; Diff.command ]
+let group = Cmd.group info [ Files.command; Apply.command; Diff.command; Show.command ]
 
 let promote =
   command_alias ~orig_name:"promotion apply" Apply.command Apply.term "promote"
