@@ -1,12 +1,17 @@
 open Stdune
 
-type t = string
+module T = struct
+  type t = Blake3_mini.Digest.t
 
-external md5_fd : Unix.file_descr -> string = "dune_md5_fd"
+  let to_string = Blake3_mini.Digest.to_hex
+  let to_dyn s = Dyn.variant "digest" [ String (to_string s) ]
+  let compare x y = Ordering.of_int (Blake3_mini.Digest.compare x y)
+end
 
-module D = Stdlib.Digest
-module Set = String.Set
-module Map = String.Map
+include T
+module C = Comparable.Make (T)
+module Set = C.Set
+module Map = C.Map
 module Metrics = Dune_metrics
 
 let file file =
@@ -21,40 +26,86 @@ let file file =
       raise (Sys_error (sprintf "%s: Permission denied" file))
     | exception exn -> reraise exn
   in
-  Exn.protectx fd ~f:md5_fd ~finally:Unix.close
+  Exn.protectx fd ~f:Blake3_mini.fd ~finally:Unix.close
 ;;
 
+let equal = Blake3_mini.Digest.equal
 let hash = Poly.hash
-let equal = String.equal
 let file p = file (Path.to_string p)
-let compare x y = Ordering.of_int (D.compare x y)
-let to_string = D.to_hex
-let to_dyn s = Dyn.variant "digest" [ String (to_string s) ]
+let from_hex s = Blake3_mini.Digest.of_hex s
 
-let from_hex s =
-  match D.from_hex s with
-  | s -> Some s
-  | exception Invalid_argument _ -> None
-;;
+module Hasher = struct
+  type t = Blake3_mini.t
 
-let string = D.string
-let to_string_raw s = s
+  let with_singleton =
+    let singleton = lazy (Blake3_mini.create ()) in
+    let in_use = ref false in
+    fun f ->
+      if !in_use
+      then
+        Code_error.raise
+          "[Hasher.with_singleton] called within argument function to \
+           [Hasher.with_singleton], which is not allowed."
+          []
+      else (
+        in_use := true;
+        let hasher = Lazy.force singleton in
+        f hasher;
+        let digest = Blake3_mini.digest hasher in
+        Blake3_mini.reset hasher;
+        in_use := false;
+        digest)
+  ;;
+end
+
+module Feed = struct
+  type hasher = Hasher.t
+  type 'a t = hasher -> 'a -> unit
+
+  let contramap a ~f hasher b = a hasher (f b)
+  let string hasher s = Blake3_mini.feed_string hasher s ~pos:0 ~len:(String.length s)
+  let bool = contramap string ~f:Bool.to_string
+  let int = contramap string ~f:Int.to_string
+
+  (* We use [No_sharing] to avoid generating different digests for inputs that
+       differ only in how they share internal values. Without [No_sharing], if a
+       command line contains duplicate flags, such as multiple occurrences of the
+       flag [-I], then [Marshal.to_string] will produce different digests depending
+       on whether the corresponding strings ["-I"] point to the same memory location
+       or to different memory locations. *)
+  let generic hasher x =
+    contramap string ~f:(fun x -> Marshal.to_string x [ No_sharing ]) hasher x
+  ;;
+
+  let list feed_x hasher xs = List.iter xs ~f:(feed_x hasher)
+  let option feed_x hasher option_x = Option.iter option_x ~f:(feed_x hasher)
+
+  let tuple2 feed_a feed_b hasher (a, b) =
+    feed_a hasher a;
+    feed_b hasher b
+  ;;
+
+  let tuple3 feed_a feed_b feed_c hasher (a, b, c) =
+    feed_a hasher a;
+    feed_b hasher b;
+    feed_c hasher c
+  ;;
+
+  let digest hasher digest = contramap string ~f:to_string hasher digest
+  let compute_digest t x = Hasher.with_singleton (fun hasher -> t hasher x)
+end
+
+let string s = Feed.compute_digest Feed.string s
+let to_string_raw s = Blake3_mini.Digest.to_binary s
 
-(* We use [No_sharing] to avoid generating different digests for inputs that
-   differ only in how they share internal values. Without [No_sharing], if a
-   command line contains duplicate flags, such as multiple occurrences of the
-   flag [-I], then [Marshal.to_string] will produce different digests depending
-   on whether the corresponding strings ["-I"] point to the same memory location
-   or to different memory locations. *)
 let generic a =
-  Metrics.Timer.record "generic_digest" ~f:(fun () ->
-    string (Marshal.to_string a [ No_sharing ]))
+  Metrics.Timer.record "generic_digest" ~f:(fun () -> Feed.compute_digest Feed.generic a)
 ;;
 
 let path_with_executable_bit =
   (* We follow the digest scheme used by Jenga. *)
   let string_and_bool ~digest_hex ~bool =
-    string (digest_hex ^ if bool then "\001" else "\000")
+    string (Blake3_mini.Digest.to_hex digest_hex ^ if bool then "\001" else "\000")
   in
   fun ~executable ~content_digest ->
     string_and_bool ~digest_hex:content_digest ~bool:executable
@@ -81,25 +132,25 @@ end
 module Path_digest_error = struct
   type nonrec t =
     | Unexpected_kind
-    | Unix_error of Dune_filesystem_stubs.Unix_error.Detailed.t
+    | Unix_error of Unix_error.Detailed.t
 end
 
 exception E of Path_digest_error.t
 
-let directory_digest_version = 2
+let directory_digest_version = 3
 
 let path_with_stats ~allow_dirs path (stats : Stats_for_digest.t) =
   let rec loop path (stats : Stats_for_digest.t) =
     match stats.st_kind with
     | S_LNK ->
-      Dune_filesystem_stubs.Unix_error.Detailed.catch
+      Unix_error.Detailed.catch
         (fun path ->
-           let contents = Unix.readlink (Path.to_string path) in
+           let contents = Path.to_string path |> Unix.readlink |> string in
            path_with_executable_bit ~executable:stats.executable ~content_digest:contents)
         path
       |> Result.map_error ~f:(fun x -> Path_digest_error.Unix_error x)
     | S_REG ->
-      Dune_filesystem_stubs.Unix_error.Detailed.catch
+      Unix_error.Detailed.catch
         (file_with_executable_bit ~executable:stats.executable)
         path
       |> Result.map_error ~f:(fun x -> Path_digest_error.Unix_error x)
