@@ -3,6 +3,7 @@ module Context_name = Context_name
 module Action_builder = Action_builder
 module Dep = Dep
 module Action = Action
+module Done_or_more_deps = Done_or_more_deps
 module Utils = Utils
 module Dir_set = Dir_set
 module Subdir_set = Subdir_set
@@ -32,12 +33,10 @@ module Fs_memo = Fs_memo
 module Execution_parameters = Execution_parameters
 module Cache_debug_flags = Cache_debug_flags
 module Report_errors_config = Report_errors_config
-module Compound_user_error = Compound_user_error
 module Reflection = Reflection
 module No_io = No_io
 module Rpc = Rpc
 module Dune_rpc_client = Dune_rpc_client
-module Action_runner = Action_runner
 module Action_exec = Action_exec
 module Running_jobs = Running_jobs
 module Rule_cache = Rule_cache
