@@ -45,4 +45,29 @@ module Dialect = Dialect
 module Lib_mode = Lib_mode
 module Melange = Melange
 module Module_name = Module_name
+module Pin_stanza = Pin_stanza
 module Preprocess = Preprocess
+module Link_flags = Link_flags
+module Dune_project = Dune_project
+module File_binding = File_binding
+module Foreign_language = Foreign_language
+module Ocaml_flags = Ocaml_flags
+module Coq_env = Coq_env
+module Rocq_env = Rocq_env
+module Menhir_env = Menhir_env
+module Dune_env = Dune_env
+module Js_of_ocaml = Js_of_ocaml
+module Menhir = Menhir
+module Rule_mode = Rule_mode
+module Rule_mode_decoder = Rule_mode_decoder
+module Mode_conf = Mode_conf
+module Oxcaml = Oxcaml
+module Modules_settings = Modules_settings
+module Stanza_pkg = Stanza_pkg
+module Package_mask = Package_mask
+
+(* CR-someday rgrinberg: perhaps wrap these under [Stanzas]? *)
+module Copy_files = Copy_files
+module Enabled_if = Enabled_if
+module Alias_conf = Alias_conf
+module Include_subdirs = Include_subdirs
