@@ -15,6 +15,8 @@ module Set : Set.S with type elt = t and type 'a map = 'a Map.t
 
 val of_string : string -> t
 val to_string : t -> string
+val hash : t -> int
+val digest_feed : t Dune_digest.Feed.t
 val arch : t
 val os : t
 val os_version : t
@@ -32,6 +34,10 @@ val build : t
 val dev : t
 val one_of : t -> t list -> bool
 
+(** The set of variable names whose values are expected to differ depending on
+    the current platform. *)
+val platform_specific : Set.t
+
 module Project : sig
   val encode : t Encoder.t
   val decode : t Decoder.t
