@@ -21,6 +21,7 @@ include (
 
 let to_opam_hash v = v
 let of_opam_hash v = v
+let of_md5 md5 = md5 |> Md5.to_hex |> OpamHash.md5
 
 let pp v =
   let s = to_string v in
@@ -28,3 +29,11 @@ let pp v =
 ;;
 
 let equal = OpamHash.equal
+let hash = Poly.hash
+
+include Comparable.Make (struct
+    type nonrec t = t
+
+    let compare x y = Ordering.of_int (OpamHash.compare x y)
+    let to_dyn = to_dyn
+  end)
