@@ -1,4 +1,4 @@
-open! Stdune
+open Stdune
 
 module Constraint : sig
   module Op : sig
@@ -25,8 +25,8 @@ type t = Dune_lang.Package_dependency.t =
 include module type of Dune_lang.Package_dependency with type t := t
 
 val opam_depend : t -> OpamParserTypes.FullPos.value
-val list_to_opam_filtered_formula : t list -> OpamTypes.filtered_formula
+val to_opam_filtered_formula : t -> OpamTypes.filtered_formula
 
 (** Attempt to interpret a [OpamTypes.filtered_formula] as a list of [t]s by
-    treating the formula as a conjunction of packages with constraints. *)
-val list_of_opam_filtered_formula : Loc.t -> OpamTypes.filtered_formula -> t list
+    treating the formula as a disjunction of packages with constraints. *)
+val list_of_opam_disjunction : Loc.t -> OpamTypes.filtered_formula -> t list
