@@ -11,6 +11,9 @@ module Spec = struct
     ; enabled_if : (Expander.t * Blang.t) list
     ; locks : Path.Set.t Action_builder.t
     ; packages : Package.Name.Set.t
+    ; timeout : (Loc.t * float) option
+    ; conflict_markers : Cram_stanza.Conflict_markers.t
+    ; setup_scripts : Path.t list
     }
 
   let make_empty ~test_name_alias =
@@ -22,6 +25,9 @@ module Spec = struct
     ; deps = []
     ; sandbox = Sandbox_config.needs_sandboxing
     ; packages = Package.Name.Set.empty
+    ; timeout = None
+    ; conflict_markers = Ignore
+    ; setup_scripts = []
     }
   ;;
 end
@@ -29,23 +35,19 @@ end
 type error = Missing_run_t of Cram_test.t
 
 let missing_run_t (error : Cram_test.t) =
-  Action_builder.fail
-    { fail =
-        (fun () ->
-          let dir =
-            match error with
-            | File _ ->
-              (* This error is impossible for file tests *)
-              assert false
-            | Dir { dir; file = _ } -> dir
-          in
-          User_error.raise
-            ~loc:(Loc.in_dir (Path.source dir))
-            [ Pp.textf
-                "Cram test directory %s does not contain a run.t file."
-                (Path.Source.to_string dir)
-            ])
-    }
+  let dir =
+    match error with
+    | File _ ->
+      (* This error is impossible for file tests *)
+      assert false
+    | Dir { dir; file = _ } -> dir
+  in
+  User_error.raise
+    ~loc:(Loc.in_dir (Path.source dir))
+    [ Pp.textf
+        "Cram test directory %s does not contain a run.t file."
+        (Path.Source.to_string dir)
+    ]
 ;;
 
 let test_rule
@@ -59,6 +61,9 @@ let test_rule
        ; locks
        ; sandbox
        ; packages = _
+       ; timeout
+       ; conflict_markers
+       ; setup_scripts
        } :
         Spec.t)
       (test : (Cram_test.t, error) result)
@@ -78,7 +83,8 @@ let test_rule
   match test with
   | Error (Missing_run_t test) ->
     (* We error out on invalid tests even if they are disabled. *)
-    Alias_rules.add sctx ~alias ~loc (missing_run_t test)
+    Action_builder.fail { fail = (fun () -> missing_run_t test) }
+    |> Alias_rules.add sctx ~alias ~loc
   | Ok test ->
     (* Morally, this is equivalent to evaluating them all concurrently and
        taking the conjunction, but we do it this way to avoid evaluating things
@@ -88,32 +94,80 @@ let test_rule
     >>= (function
      | false -> Alias_rules.add_empty sctx ~alias ~loc
      | true ->
+       let prefix_with, _ = Path.Build.extract_build_context_dir_exn dir in
+       let script = Path.Build.append_source prefix_with (Cram_test.script test) in
+       let base_path =
+         Path.Build.append_source
+           prefix_with
+           (let path =
+              match test with
+              | File f -> f
+              | Dir d -> d.dir
+            in
+            let dir = Path.Source.parent_exn path in
+            let basename = Path.Source.basename path in
+            Path.Source.relative dir (".cram." ^ basename))
+       in
+       let script_sh = Path.Build.relative base_path "cram.sh" in
+       let output = Path.Build.relative base_path "cram.out" in
+       let* () =
+         (let open Action_builder.O in
+          let+ () = Action_builder.path (Path.build script) in
+          Cram_exec.make_script
+            ~src:(Path.build script)
+            ~script:script_sh
+            ~conflict_markers
+          |> Action.Full.make)
+         |> Action_builder.with_file_targets ~file_targets:[ script_sh ]
+         |> Super_context.add_rule sctx ~dir ~loc
+       in
+       let* () =
+         (let open Action_builder.O in
+          let+ () = Action_builder.all_unit deps
+          and+ () = Action_builder.path (Path.build script_sh)
+          and+ () =
+            match test with
+            | File _ -> Action_builder.return ()
+            | Dir { dir; file } ->
+              let file = Path.Build.append_source prefix_with file |> Path.build in
+              let deps =
+                Path.Build.append_source prefix_with dir
+                |> Path.build
+                |> Source_deps.files_with_filter ~filter:(fun file' ->
+                  not (Path.equal file file'))
+              in
+              let+ (_ : Path.Set.t) = Action_builder.dyn_memo_deps deps in
+              ()
+          and+ () = Action_builder.paths setup_scripts
+          and+ locks = locks >>| Path.Set.to_list in
+          Cram_exec.run
+            ~src:(Path.build script)
+            ~dir:
+              (Path.build
+                 (match test with
+                  | File _ -> Path.Build.parent_exn script
+                  | Dir d -> Path.Build.append_source prefix_with d.dir))
+            ~script:(Path.build script_sh)
+            ~output
+            ~timeout
+            ~setup_scripts
+          |> Action.Full.make ~locks ~sandbox)
+         |> Action_builder.with_file_targets ~file_targets:[ output ]
+         |> Super_context.add_rule sctx ~dir ~loc
+       in
        Alias_rules.add sctx ~alias ~loc
        @@
        let open Action_builder.O in
-       let prefix_with, _ = Path.Build.extract_build_context_dir_exn dir in
-       let script = Path.Build.append_source prefix_with (Cram_test.script test) in
-       let+ () = Action_builder.path (Path.build script)
-       and+ () = Action_builder.all_unit deps
-       and+ () =
-         match test with
-         | File _ -> Action_builder.return ()
-         | Dir { dir; file = _ } ->
-           let deps =
-             Path.Build.append_source prefix_with dir |> Path.build |> Source_deps.files
-           in
-           let+ (_ : Path.Set.t) = Action_builder.dyn_memo_deps deps in
-           ()
-       and+ locks = locks >>| Path.Set.to_list in
+       let+ () = List.map ~f:Path.build [ script; output ] |> Action_builder.paths in
        Action.progn
-         [ Cram_exec.action (Path.build script)
+         [ Cram_exec.diff ~src:(Path.build script) ~output:(Path.build output)
          ; Promote.Diff_action.diff
              ~optional:true
              ~mode:Text
              (Path.build script)
              (Path.Build.extend_basename script ~suffix:".corrected")
          ]
-       |> Action.Full.make ~locks ~sandbox)
+       |> Action.Full.make)
 ;;
 
 let collect_stanzas =
@@ -147,10 +201,10 @@ let rules ~sctx ~dir tests =
   let* stanzas = collect_stanzas ~dir
   and* with_package_mask =
     Dune_load.mask ()
+    >>| Only_packages.enumerate
     >>| function
-    | None -> fun _packages f -> f ()
-    | Some only ->
-      let only = Package.Name.Set.of_keys only in
+    | `All -> fun _packages f -> f ()
+    | `Set only ->
       fun packages f ->
         Memo.when_
           (Package.Name.Set.is_empty packages
@@ -239,6 +293,29 @@ let rules ~sctx ~dir tests =
                 | Some (p : Package.t) ->
                   Package.Name.Set.add acc.packages (Package.name p)
               in
+              let timeout =
+                Option.merge
+                  acc.timeout
+                  stanza.timeout
+                  ~f:(Ordering.min (fun x y -> Float.compare (snd x) (snd y)))
+              in
+              let conflict_markers =
+                Option.value ~default:acc.conflict_markers stanza.conflict_markers
+              in
+              let setup_scripts =
+                let more_current_scripts =
+                  List.map stanza.setup_scripts ~f:(fun (_loc, script) ->
+                    (* Handle both relative and absolute paths *)
+                    if Filename.is_relative script
+                    then Path.build (Path.Build.relative dir script)
+                    else Path.external_ (Path.External.of_string script))
+                in
+                (* This is a silly way to dedupe, but we aim to preserve the
+                   order as much as possible. *)
+                more_current_scripts
+                @ List.filter acc.setup_scripts ~f:(fun x ->
+                  not (List.mem more_current_scripts x ~equal:Path.equal))
+              in
               ( runtest_alias
               , { acc with
                   enabled_if
@@ -248,6 +325,9 @@ let rules ~sctx ~dir tests =
                 ; extra_aliases
                 ; packages
                 ; sandbox
+                ; timeout
+                ; conflict_markers
+                ; setup_scripts
                 } ))
       in
       let extra_aliases =
