@@ -2,7 +2,7 @@ open Import
 
 (** Tie the knot between [Dune_engine] and [Dune_rules]. *)
 val init
-  :  stats:Dune_stats.t option
+  :  stats:Dune_trace.t option
   -> sandboxing_preference:Sandbox_mode.t list
   -> cache_config:Dune_cache.Config.t
   -> cache_debug_flags:Dune_engine.Cache_debug_flags.t
