@@ -13,6 +13,7 @@ val lib
   -> stdlib:Ocaml_stdlib.t option
   -> lib_name:Lib_name.Local.t
   -> implements:bool
+  -> has_instances:bool
   -> modules:Module.t Module_trie.t
   -> t
 
@@ -32,6 +33,7 @@ val exe_unwrapped : Module.t Module_trie.t -> obj_dir:Path.Build.t -> t
 val make_wrapped
   :  obj_dir:Path.Build.t
   -> modules:Module.t Module_trie.t
+  -> has_instances:bool
   -> [ `Exe | `Melange ]
   -> t
 
@@ -55,6 +57,12 @@ val virtual_module_names : t -> Module_name.Path.Set.t
 
 val wrapped : t -> Wrapped.t
 val source_dirs : t -> Path.Set.t
+val compat_for_exn : t -> Module.t -> Module.t
+
+(** List of entry modules visible to users of the library. For wrapped
+    libraries, this is always one module. For unwrapped libraries, this could
+    be more than one. *)
+val entry_modules : t -> Module.t list
 
 module With_vlib : sig
   type modules := t
@@ -72,7 +80,6 @@ module With_vlib : sig
     -> (Module.t list, [ `Parent_cycle ]) result
 
   val find : t -> Module_name.t -> Module.t option
-  val compat_for_exn : t -> Module.t -> Module.t
   val impl_only : t -> Module.t list
 
   (** A set of modules from a single module. Not suitable for single module exe as
@@ -96,15 +103,6 @@ module With_vlib : sig
   (** Returns all the compatibility modules. *)
   val wrapped_compat : t -> Module.Name_map.t
 
-  (** List of entry modules visible to users of the library. For wrapped
-      libraries, this is always one module. For unwrapped libraries, this could be
-      more than one. *)
-  val entry_modules : t -> Module.t list
-
-  (** Returns the main module name if it exists. It exist for libraries with
-      [(wrapped true)] or one module libraries. *)
-  val main_module_name : t -> Module_name.t option
-
   val version_installed : t -> src_root:Path.t -> install_dir:Path.t -> t
   val alias_for : t -> Module.t -> Module.t list
   val local_open : t -> Module.t -> Module_name.t list
